require "bluehouselab/sms/version"
require "httparty"
require "json"

module Bluehouselab
  class Sms

    ROUTES = {
      :lms => "sendlms",
      :sms => "sendsms",
      :result => "result"
    }

    HEADERS = { "Content-Type" => "application/json;charset=utf-8" }

    HTTP_ERRORS = {
      400 => "Bad request, check the values sent to the server",
      401 => "Unauthorized, check api key and app_id",
      402 => "Insufficient credit, please check your account balance",
      403 => "Forbidden, connect via HTTPS",
      404 => "Not found, delivery id or resource",
      406 => "Not acceptable, check the JSON payload isn't malformed",
      410 => "API deprecated",
      412 => "Precondition Failed, check the values sent to the server",
      500 => "Internal server error",
      503 => "Service unavailable"
    }

    SEND_RESULT_ERRORS = {
      -1 => "Cannot confirm at this time, please retry",
      10000 => "Message failed",
      10001 => "Device cannot receive messages",
      10002 => "NPDB error",
      10003 => "Service temporarily unavailable",
      10004 => "Receiving device error",
      10005 => "System error",
      10006 => "Message request time limit has expired"
    }

    def initialize(options)
      raise ArgumentError.new "api_key cannot be nil" unless options[:api_key]
      raise ArgumentError.new "app_id cannot be nil" unless options[:app_id]
      raise ArgumentError.new "sender cannot be nil" unless options[:sender]
      @api_key = options[:api_key]
      @app_id = options[:app_id]
      @sender = options[:sender]
      @base_url = "https://#{@app_id}:#{@api_key}@api.bluehouselab.com/smscenter/v1.0/"
    end

    def send_message(options)
      raise ArgumentError.new "send_message options requires a type either :sms or :lms" unless options[:type]
      raise ArgumentError.new "send_message options requires an array of receiver phone numbers" unless options[:receivers] && options[:receivers].is_a?(Array)
      raise ArgumentError.new "send_message options requires content" unless options[:content]

      type = options[:type].nil? ? :sms : options[:type]

      raise ArgumentError.new "send_message options[:type] must be one of :sms or :lms" unless type == :sms || type == :lms
      raise "content too long for type :sms" unless (options[:content].bytesize <= 80 && type == :sms) || ( type == :lms)

      res = send type, options[:receivers], options[:content], options[:subject]
      parse_send_response res.code, res.parsed_response
    end

    def result(request_id)
      res = HTTParty.get "#{@base_url}#{ROUTES[:result]}/#{request_id}", { :headers => HEADERS }
      parse_result_response res.code, res.parsed_response
    end

    private

    def parse_error_and_code(code)
      if code >= 400
        error = HTTP_ERRORS[code]
        success = false
      else
        error = nil
        success = true
      end
      { :error => error, :success => success, :status_code => code }
    end

    def parse_result_response(code,parsed_body)
      error_and_code = parse_error_and_code(code)
      if error_and_code[:success]
        if parsed_body['status'] == 0
          error_and_code.merge! :sent_time => Time.parse(parsed_body['sent_time'])
        else
          error = SEND_RESULT_ERRORS[parsed_body['status']]
          error_and_code.merge!({ :error => error })
        end
      else
        error_and_code
      end
    end

    def parse_send_response(code,parsed_body)
      error_and_code = parse_error_and_code(code)
      { :message_ids => code >= 400 ? [] : parsed_body["sent"] }.merge! error_and_code
    end

    def send(type,receivers,content,subject)
      body = {
        :sender => @sender,
        :receivers =>  receivers,
        :content => content
      }

      if subject && type == :lms
        body.merge! :subject => subject
      end

      HTTParty.post "#{@base_url}#{ROUTES[type]}", :body => body.to_json, :headers => HEADERS
    end

  end
end
